Package: mfdb
Type: Package
Title: MareFrame DB Querying Library
Encoding: UTF-8
Version: 7.4-0
Date: 2026-01-24
Authors@R: c(person("Jamie", "Lentin", email="lentinj@shuttlethread.com",role = c("aut", "cre", "cph")),
             person("Bjarki Thor", "Elvarsson", email="bjarki.elvarsson@hafogvatn.is",role = c("aut")))
Maintainer: Jamie Lentin <lentinj@shuttlethread.com>
Description: Creates and manages a PostgreSQL database suitable for storing fisheries data
             and aggregating ready for use within a Gadget <https://gadget-framework.github.io/gadget2/> model.
             See <https://mareframe.github.io/mfdb/> for more information.
License: GPL-3
Depends: R (>= 4.2.0)
Imports: logging (>= 0.7-103), DBI (>= 0.3.1), duckdb (>= 0.2.5),
        getPass (>= 0.1-1), rlang (>= 0.4.0)
Suggests: dplyr (>= 0.8.3), dbplyr (>= 2.0.0), knitr, rmarkdown,
        RPostgres (>= 1.3.0), RSQLite, unittest (>= 1.5-0)
VignetteBuilder: knitr
LazyData: true
NeedsCompilation: no
Packaged: 2026-01-24 10:09:58 UTC; lentinj
Author: Jamie Lentin [aut, cre, cph],
  Bjarki Thor Elvarsson [aut]
Repository: CRAN
Date/Publication: 2026-01-24 11:50:02 UTC
Built: R 4.6.0; ; 2026-04-25 20:21:50 UTC; unix
